# frozen_string_literal: true

# https://en.wikipedia.org/wiki/Check_digit
# SSCC codes (18 digits) are treated as invalid

module ReferenceValidator
  class ValidateEan
    attr_reader :code

    DEFAULT_CODE_LENGTH = 18
    MAX_CODE_LENGTH = 13

    def initialize(code)
      @code = code
    end

    def validate!
      return true if code.present? && format_valid? && checksum_valid?

      false
    end

    private

    def format_valid?
      return false unless code.length == MAX_CODE_LENGTH

      code.match?(/\A\d{1,#{DEFAULT_CODE_LENGTH}}\z/)
    end

    def checksum_valid?
      digits    = Array.new(DEFAULT_CODE_LENGTH - code.length, 0) + code.split('').map(&:to_i)
      remainder = data_digits_sum(digits[0..-2]) % 10

      digits[-1] == (remainder.zero? ? remainder : 10 - remainder)
    end

    def data_digits_sum(data_digits)
      data_digits.each_with_index.reduce(0) do |sum, (digit, index)|
        sum + ((index % 2).zero? ? digit * 3 : digit)
      end
    end
  end
end
